@@ -1,4 +1,7 @@
+from __future__ import unicode_literals
+import warnings
 
+import django
 from django.core.exceptions import ImproperlyConfigured
 from django.db import models
 
@@ -6,25 +9,37 @@ from .utils import get_content_type, get_content_types
 
 
 class PolymorphicQuerySet(models.query.QuerySet):
-    def select_subclasses(self, *subclasses):
+    def select_subclasses(self, *args):
         self.type_cast = True
         content_type_field_name = self.model.CONTENT_TYPE_FIELD
         lookups = set([content_type_field_name])
         opts = self.model._meta
         accessors = opts._subclass_accessors
-        if subclasses:
-            sub_subclasses = set()
-            for subclass in subclasses:
+        if args:
+            subclasses = set()
+            # Collect all subclasses
+            for subclass in args:
                 if not issubclass(subclass, self.model):
                     raise TypeError("%r is not a subclass of %r" % (subclass,
                                                                     self.model))
-                sub_subclasses.update(subclass._meta._subclass_accessors.iterkeys())
-                lookups.add(accessors[subclass][2])
-            content_types = [ct.pk for cls, ct in get_content_types(sub_subclasses).iteritems()]
-            filters = {"%s__in" % content_type_field_name: content_types}
+                subclasses.update(subclass._meta._subclass_accessors.keys())
+            # Collect all `select_related` required lookups
+            for subclass in subclasses:
+                # Avoid collecting ourself and proxy subclasses
+                subclass_lookup = accessors[subclass][2]
+                if subclass_lookup:
+                    lookups.add(subclass_lookup)
+            # Fetch associated `ContentType` instances for filtering
+            content_types = get_content_types(subclasses)
+            filters = {"%s__in" % content_type_field_name:
+                       tuple(ct.pk for ct in content_types.values())}
             qs = self.filter(**filters)
         else:
-            lookups.update(accessor[2] for accessor in accessors.itervalues())
+            # Collect all `select_related` required lookups
+            for accessor in accessors.values():
+                # Avoid collecting ourself and proxy subclasses
+                if accessor[2]:
+                    lookups.add(accessor[2])
             qs = self
         return qs.select_related(*lookups)
 
@@ -33,9 +48,9 @@ class PolymorphicQuerySet(models.query.QuerySet):
         model_content_type = get_content_type(self.model)
         return self.filter(**{content_type_field_name: model_content_type})
 
-    def _clone(self, **kwargs):
+    def _clone(self, *args, **kwargs):
         kwargs.update(type_cast=getattr(self, 'type_cast', False))
-        return super(PolymorphicQuerySet, self)._clone(**kwargs)
+        return super(PolymorphicQuerySet, self)._clone(*args, **kwargs)
 
     def iterator(self):
         iterator = super(PolymorphicQuerySet, self).iterator()
@@ -47,6 +62,7 @@ class PolymorphicQuerySet(models.query.QuerySet):
             for obj in iterator:
                 yield obj
 
+
 class PolymorphicManager(models.Manager):
     use_for_related_fields = True
 
@@ -58,11 +74,23 @@ class PolymorphicManager(models.Manager):
                                        'on `BasePolymorphicModel` subclasses.')
         return super(PolymorphicManager, self).contribute_to_class(model, name)
 
-    def get_query_set(self):
+    def get_queryset(self):
         return PolymorphicQuerySet(self.model, using=self._db)
 
+    if django.VERSION < (1, 8):
+        def get_query_set(self):
+            if django.VERSION >= (1, 6):
+                warnings.warn(
+                    "`PolymorphicManager.get_query_set` is deprecated, use "
+                    "`get_queryset` instead",
+                    DeprecationWarning if django.VERSION >= (1, 7)
+                        else PendingDeprecationWarning,
+                    stacklevel=2
+                )
+            return PolymorphicManager.get_queryset(self)
+
     def select_subclasses(self, *subclasses):
-        return self.get_query_set().select_subclasses(*subclasses)
+        return self.get_queryset().select_subclasses(*subclasses)
 
     def exclude_subclasses(self):
-        return self.get_query_set().exclude_subclasses()
+        return self.get_queryset().exclude_subclasses()
